@@ -1,5 +1,6 @@
+import array
 from operator import and_
-from struct import calcsize, unpack, iter_unpack
+from struct import calcsize, iter_unpack
 from warnings import warn
 import os
 import re
@@ -28,8 +29,10 @@ class FlowData(object):
 
     :param filename_or_handle: a path string or a file handle for an FCS file
     :param ignore_offset_error: option to ignore data offset error (see above note), default is False
+    :param only_text: option to only read the "text" segment of the FCS file without loading event data,
+        default is False
     """
-    def __init__(self, filename_or_handle, ignore_offset_error=False):
+    def __init__(self, filename_or_handle, ignore_offset_error=False, only_text=False):
         if isinstance(filename_or_handle, basestring):
             self._fh = open(str(filename_or_handle), 'rb')
         else:
@@ -85,12 +88,15 @@ class FlowData(object):
         if d_stop > self.file_size:
             raise EOFError("FCS header indicates data section greater than file size")
 
-        self.events = self.__parse_data(
-            self.cur_offset,
-            d_start,
-            d_stop,
-            self.text
-        )
+        if only_text:
+            self.events = None
+        else:
+            self.events = self.__parse_data(
+                self.cur_offset,
+                d_start,
+                d_stop,
+                self.text
+            )
 
         self.channels = self._parse_channels()
 
@@ -136,30 +142,39 @@ class FlowData(object):
 
     def __parse_text(self, offset, start, stop):
         """return parsed text segment of FCS file"""
-        text = self.__read_bytes(offset, start, stop)
+        num_items = (stop - start + 1)
+        self._fh.seek(offset + start)
+        tmp = array.array('b')
+        tmp.fromfile(self._fh, int(num_items))
+        tmp = tmp.tobytes()
+
         try:
             # try UTF-8 first
-            text = text.decode()
+            tmp = tmp.decode()
         except UnicodeDecodeError:
             # next best guess is Latin-1, if not that either, we throw the exception
-            text = text.decode("ISO-8859-1")
-        return self.__parse_pairs(text)
+            tmp = tmp.decode("ISO-8859-1")
+        return self.__parse_pairs(tmp)
 
     def __parse_analysis(self, offset, start, stop):
         """return parsed analysis segment of FCS file"""
         if start == stop:
             return {}
         else:
-            text = self.__read_bytes(offset, start, stop)
+            num_items = (stop - start + 1)
+            self._fh.seek(offset + start)
+            tmp = array.array('b')
+            tmp.fromfile(self._fh, int(num_items))
+            tmp = tmp.tobytes()
 
             try:
                 # try UTF-8 first
-                text = text.decode()
+                tmp = tmp.decode()
             except UnicodeDecodeError:
                 # next best guess is Latin-1, if not that either, we throw the exception
-                text = text.decode("ISO-8859-1")
+                tmp = tmp.decode("ISO-8859-1")
 
-            return self.__parse_pairs(text)
+            return self.__parse_pairs(tmp)
 
     def __parse_data(self, offset, start, stop, text):
         """
@@ -252,20 +267,16 @@ class FlowData(object):
                 data_type_size = bit_width[0] / 8
                 num_items, stop = self.__calc_data_item_count(start, stop, data_type_size)
 
-                # unpack to a list
-                tmp = unpack(
-                    '%s%d%s' %
-                    (
-                        order,
-                        num_items,
-                        self.__format_integer(bit_width[0])
-                    ),
-                    self.__read_bytes(offset, start, stop)
-                )
+                self._fh.seek(offset + start)
+                tmp = array.array(self.__format_integer(bit_width[0]))
+                tmp.fromfile(self._fh, int(num_items))
+                if order == '>':
+                    tmp.byteswap()
 
             # parameter sizes are different
             # e.g. 8, 8, 16, 8, 32 ...
             else:
+                # can't use array for heterogeneous bit widths
                 tmp = self.__extract_var_length_int(bit_width, offset, order, start, stop)
 
         else:  # non standard bit width...  Does this happen?
@@ -278,6 +289,8 @@ class FlowData(object):
         for cur_width in bit_width:
             data_format += '%s' % self.__format_integer(cur_width)
 
+        # array module doesn't have a function to heterogeneous bit widths,
+        # so fall back to the slower unpack approach
         tuple_tmp = iter_unpack(data_format, self.__read_bytes(offset, start, stop))
         tmp = [ti for t in tuple_tmp for ti in t]
         return tmp
@@ -287,10 +300,11 @@ class FlowData(object):
         data_type_size = calcsize(data_type)
         num_items, stop = self.__calc_data_item_count(start, stop, data_type_size)
 
-        tmp = unpack(
-            '%s%d%s' % (order, num_items, data_type),
-            self.__read_bytes(offset, start, stop)
-        )
+        self._fh.seek(offset + start)
+        tmp = array.array(data_type)
+        tmp.fromfile(self._fh, int(num_items))
+        if order == '>':
+            tmp.byteswap()
         return tmp
 
     @staticmethod
@@ -368,6 +382,12 @@ class FlowData(object):
         :param extra_non_standard: an optional dictionary for adding extra non-standard keywords/values
         :return: None
         """
+        if self.events is None:
+            raise AttributeError(
+                "FlowData instance does not contain event data. This might"
+                "occur if the FCS file was read with the only_text=True option."
+            )
+
         pnn_labels = [''] * len(self.channels)
         pns_labels = [''] * len(self.channels)
 
